/////////////////////////////////////////////////////////////////////////////
// Name:        WxPadFrame.cpp
// Purpose:     Simple version of a notepad replacement
// Author:      
// Modified by: Biswapesh Chattopadhyay
// Created:     2001 05 11
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
#pragma implementation
#pragma interface
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "WxPadFrame.h"

BEGIN_EVENT_TABLE(WxPadFrame, wxFrame)
    // File functions
    EVT_MENU(IDM_FILE_NEW,  WxPadFrame::OnNew)
    EVT_MENU(IDM_FILE_OPEN,  WxPadFrame::OnOpen)
    EVT_MENU(IDM_FILE_SAVE,  WxPadFrame::OnSave)
    EVT_MENU(IDM_FILE_SAVE_AS,  WxPadFrame::OnSaveAs)
    EVT_MENU(IDM_FILE_QUIT,  WxPadFrame::OnExit)
    // Edit functions
    EVT_MENU(IDM_EDIT_COPY, WxPadFrame::OnCopy)
    EVT_MENU(IDM_EDIT_PASTE, WxPadFrame::OnPaste)
    EVT_MENU(IDM_EDIT_CUT, WxPadFrame::OnCut)
    EVT_MENU(IDM_EDIT_SELECT_ALL, WxPadFrame::OnSelectAll)
    EVT_MENU(IDM_EDIT_UNDO, WxPadFrame::OnUndo)
    EVT_MENU(IDM_EDIT_REDO, WxPadFrame::OnUndo)
    // Help functions
    EVT_MENU(IDM_HELP_ABOUT, WxPadFrame::OnAbout)
    // Window functions
    EVT_CLOSE(WxPadFrame::OnExit)
END_EVENT_TABLE()

WxPadFrame::WxPadFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
    : wxFrame((wxFrame *)NULL, -1, title, pos, size, (wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)),
              text(NULL), fileName(wxEmptyString)
{
#ifdef __WXMAC__
    WxPadApp::s_macAboutMenuItemId = IDM_HELP_ABOUT;
#endif
    SetIcon(wxICON(icon));
    
    // File menu
    wxMenu *fileMenu = new wxMenu;
    fileMenu->Append(IDM_FILE_NEW,      "&New",     "Create a new file");
    fileMenu->Append(IDM_FILE_OPEN,     "&Open",    "Open an existing file");
    fileMenu->Append(IDM_FILE_SAVE,     "&Save",    "Save the current file");
    fileMenu->Append(IDM_FILE_SAVE_AS,  "Save &As", "Save as a different name");
    fileMenu->Append(IDM_FILE_QUIT,     "E&xit",    "Quit this program");
    // Edit menu
    wxMenu *editMenu = new wxMenu;
    editMenu->Append(IDM_EDIT_COPY,     "&Copy",    "Copy selected text");
    editMenu->Append(IDM_EDIT_PASTE,    "&Paste",   "Paste from clipboard");
    editMenu->Append(IDM_EDIT_CUT,      "C&ut",     "Cut selected text");
    editMenu->Append(IDM_EDIT_SELECT_ALL, "Select &All", "Select everything");
    editMenu->Append(IDM_EDIT_UNDO,     "&Undo",    "Undo last operation");
    editMenu->Append(IDM_EDIT_REDO,     "&Redo",    "Redo last Undo");
    // Help menu
    wxMenu *helpMenu = new wxMenu;
    helpMenu->Append(IDM_HELP_ABOUT,    "&About",   "About this program");
    
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, "&File");
    menuBar->Append(editMenu, "&Edit");
    menuBar->Append(helpMenu, "&Help");
    SetMenuBar(menuBar);
    
    // Create the editor component
    text = new WxPadTextCtrl(this);
    // Set the statusbar
#if wxUSE_STATUSBAR
    CreateStatusBar();
    SetStatusText("Untitled");
#endif
}

// event handlers

// File functions
void WxPadFrame::OnNew(wxCommandEvent &event)
{
    text->Clear();
    SetTitle("Untitled");
    fileName = wxEmptyString;
}

void WxPadFrame::OnOpen(wxCommandEvent &event)
{
    OpenFile();
}

void WxPadFrame::OnSave(wxCommandEvent &event)
{
    SaveFile();
}

void WxPadFrame::OnSaveAs(wxCommandEvent &event)
{
    SaveFile(true);
}

void WxPadFrame::OnExit(wxCommandEvent &event)
{
    if (OkToClose())
        this->Destroy();
}

// Edit functions
void WxPadFrame::OnCopy(wxCommandEvent &event)
{
    if (text->CanCopy())
        text->Copy();
}

void WxPadFrame::OnPaste(wxCommandEvent &event)
{
    if (text->CanPaste())
        text->Paste();
}

void WxPadFrame::OnCut(wxCommandEvent &event)
{
    if (text->CanCut())
        text->Cut();
}

void WxPadFrame::OnSelectAll(wxCommandEvent &event)
{
    text->SetSelection(0, text->GetLastPosition());
}

void WxPadFrame::OnUndo(wxCommandEvent &event)
{
    if (text->CanUndo())
        text->Undo();
}

void WxPadFrame::OnRedo(wxCommandEvent &event)
{
    if (text->CanRedo())
        text->Redo();
}

// Help functions
void WxPadFrame::OnAbout(wxCommandEvent &event)
{
    wxMessageBox(wxT("WxPad\nBy Biswapesh Chattopadhyay"), "About WxPad", (wxOK | wxICON_INFORMATION), this);
}

// Private function implementation
void WxPadFrame::OpenFile()
{
    wxFileDialog *fileDialog = new wxFileDialog(this, wxT("Open File"), wxT(""), wxT(""), wxT("*.txt"), wxOPEN);
    if (fileDialog->ShowModal() == wxID_OK)
    {
        fileName = fileDialog->GetFilename();
        if (wxFileExists(fileName))
        {
            if (!text->LoadFile(fileName))
            {
                wxMessageBox(wxT("Failed to load file!"), fileName, (wxOK | wxICON_EXCLAMATION), this);
                fileName = wxEmptyString;
            }
            else
                SetTitle(fileName);
        }
        else
        {
            wxMessageBox(wxT("File does not exist!"), fileName, (wxOK | wxICON_EXCLAMATION), this);
            fileName = wxEmptyString;
        }
    }
}

void WxPadFrame::SaveFile(bool isSaveAs)
{
    if ((fileName.IsEmpty()) || (isSaveAs))
    {
        wxFileDialog *fileDialog = new wxFileDialog(this, wxT("Save File"), wxT(""), wxT(""), wxT("*.txt"), (wxSAVE | wxOVERWRITE_PROMPT));
        if (fileDialog->ShowModal() == wxID_OK)
            fileName = fileDialog->GetFilename();
    }
    if (!fileName.IsEmpty())
    {
        if (!text->SaveFile(fileName))
        {
            wxMessageBox(wxT("Failed saving file!"), fileName, (wxOK | wxICON_EXCLAMATION), this);
            fileName = wxEmptyString;
        }
    }
    SetTitle(fileName);
}

bool WxPadFrame::OkToClose()
{
    if (text->IsModified())
    {
        wxString msgBoxTitle;
        if (fileName.IsEmpty())
            msgBoxTitle = wxT("Untitled");
        else
            msgBoxTitle = fileName;
        if (wxMessageBox(wxT("Document unsaved. Quit anyway ?"), msgBoxTitle
            , (wxYES_NO | wxCENTER | wxICON_QUESTION), this) != wxYES)
            return FALSE;
    }
    return TRUE;
}
